@@ -1,6 +1,5 @@
 use std::{
-    io,
-    mem,
+    io, mem,
     net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr},
     os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd},
     pin::Pin,
@@ -22,9 +21,7 @@ use tokio_tfo::TfoStream;
 use crate::net::{
     sys::{set_common_sockopt_after_connect, set_common_sockopt_for_connect, socket_bind_dual_stack},
     udp::{BatchRecvMessage, BatchSendMessage},
-    AcceptOpts,
-    AddrFamily,
-    ConnectOpts,
+    AcceptOpts, AddrFamily, ConnectOpts,
 };
 
 /// A `TcpStream` that supports TFO (TCP Fast Open)
@@ -216,12 +213,18 @@ pub fn set_tcp_fastopen<S: AsRawFd>(socket: &S) -> io::Result<()> {
 }
 
 fn create_mptcp_socket(bind_addr: &SocketAddr) -> io::Result<TcpSocket> {
+    // https://www.kernel.org/doc/html/next/networking/mptcp.html
+
     unsafe {
         let family = match bind_addr {
             SocketAddr::V4(..) => libc::AF_INET,
             SocketAddr::V6(..) => libc::AF_INET6,
         };
         let fd = libc::socket(family, libc::SOCK_STREAM, libc::IPPROTO_MPTCP);
+        if fd < 0 {
+            let err = io::Error::last_os_error();
+            return Err(err);
+        }
         let socket = Socket::from_raw_fd(fd);
         socket.set_nonblocking(true)?;
         Ok(TcpSocket::from_raw_fd(socket.into_raw_fd()))
